@@ -1,30 +1,53 @@
 package Kelp::Module::Symbiosis;
 
-our $VERSION = '1.01';
+our $VERSION = '1.10';
 
 use Kelp::Base qw(Kelp::Module);
 use Plack::App::URLMap;
 use Carp;
-use Scalar::Util qw(blessed);
+use Scalar::Util qw(blessed refaddr);
 
 attr "-mounted" => sub { {} };
+attr "-loaded" => sub { {} };
 
 sub mount
 {
 	my ($self, $path, $app) = @_;
 	my $mounted = $self->mounted;
 
-	carp "Overriding mounting point $path"
-		unless !exists $mounted->{$path};
+	if (!ref $app && $app) {
+		my $loaded = $self->loaded;
+		croak "Symbiosis: cannot mount $app, because no such name was loaded"
+			unless $loaded->{$app};
+		$app = $loaded->{$app};
+	}
+
+	carp "Symbiosis: overriding mounting point $path"
+		if exists $mounted->{$path};
 	$mounted->{$path} = $app;
 	return scalar keys %{$mounted};
 }
 
+sub _link
+{
+	my ($self, $name, $app, $mount) = @_;
+	my $loaded = $self->loaded;
+
+	warn "Symbiosis: overriding module name $name"
+		if exists $loaded->{$name};
+	$loaded->{$name} = $app;
+
+	if ($mount) {
+		$self->mount($mount, $app);
+	}
+	return scalar keys %{$loaded};
+}
+
 sub run_all
 {
 	my ($self) = shift;
 
-	warn __PACKAGE__ . '->run_all is deprecated, please use ' . __PACKAGE__ . ' run instead';
+	warn 'Symbiosis: run_all method is deprecated, use run instead';
 	return $self->run(@_);
 }
 
@@ -32,19 +55,25 @@ sub run
 {
 	my ($self) = shift;
 	my $psgi_apps = Plack::App::URLMap->new;
+	my %addrs;    # apps keyed by refaddr
 
-	my $error = "Cannot start the ecosystem:";
+	my $error = "Symbiosis: cannot start the ecosystem because";
 	while (my ($path, $app) = each %{$self->mounted}) {
 		if (blessed $app) {
 			croak "$error application mounted under $path cannot run()"
 				unless $app->can("run");
-			$psgi_apps->map($path, $app->run(@_));
+
+			# cache the ran application so that it won't be ran twice
+			my $addr = refaddr $app;
+			my $ran = $addrs{$addr} //= $app->run(@_);
+
+			$psgi_apps->map($path, $ran);
 		}
 		elsif (ref $app eq 'CODE') {
 			$psgi_apps->map($path, $app);
 		}
 		else {
-			croak "$error mount point $path is not an object";
+			croak "$error mount point $path is neither an object nor a coderef";
 		}
 	}
 
@@ -54,8 +83,14 @@ sub run
 sub build
 {
 	my ($self, %args) = @_;
-	if (!exists $args{automount} || $args{automount}) {
-		$self->mount("/", $self->app);
+	$args{mount} //= '/'
+		unless exists $args{mount};
+
+	warn 'Symbiosis: automount configuration is deprecated, use mount instead'
+		if exists $args{automount};
+
+	if ($args{mount} && (!exists $args{automount} || $args{automount})) {
+		$self->mount($args{mount}, $self->app);
 	}
 
 	$self->register(
@@ -78,16 +113,21 @@ Kelp::Module::Symbiosis - Manage an entire ecosystem of Plack organisms under Ke
 	modules => [qw/Symbiosis SomeSymbioticModule/],
 	modules_init => {
 		Symbiosis => {
-			automount => 0, # boolean, defaults to 1
+			mount => '/kelp', # a path to mount Kelp main instance
+		},
+		SomeSymbioticModule => {
+			mount => '/elsewhere', # a path to mount SomeSymbioticModule
 		},
 	},
 
-	# in kelp application
-	$kelp->symbiosis->mount('/app-path' => $kelp); # only required if config 'automount' is explicitly false
-	$kelp->symbiosis->mount('/other-path' => $kelp->some_symbiotic_module);
+	# in kelp application - can be skipped if all mount paths are specified in config above
+	my $symbiosis = $kelp->symbiosis;
+	$symbiosis->mount('/app-path' => $kelp);
+	$symbiosis->mount('/other-path' => $kelp->module_method);
+	$symbiosis->mount('/other-path' => 'module_name'); # alternative - finds a module by name
 
 	# in psgi script
-	my $app = MyApp->new();
+	my $app = KelpApp->new();
 	$app->run_all; # instead of run
 
 =head1 DESCRIPTION
@@ -121,13 +161,6 @@ The main functional reason to use this module is the ability to access the Kelp
 		... # handle another app's signal
 	}
 
-	sub build {
-		my ($kelp) = @_;
-
-		my $another_app = $kelp->get_another_app;
-		$kelp->symbiosis->mount('/app' => $another_app);
-	}
-
 =head2 What can be mounted?
 
 The sole requirement for a module to be mounted into Symbiosis is its ability to I<run()>, returning the psgi application. A module also needs to be a blessed reference, of course. Fun fact: Symbiosis module itself meets that requirements, so one symbiotic app can be mounted inside another.
@@ -139,7 +172,7 @@ Whichever it is, it should be a psgi application ready to be ran by the server,
 For very simple use cases, this will work though:
 
 	# in application build method
-	my $some_app = SomePlackApp->new()->to_app;
+	my $some_app = SomePlackApp->new->to_app;
 	$self->symbiosis->mount('/path', $some_app);
 
 =head1 METHODS
@@ -148,23 +181,51 @@ For very simple use cases, this will work though:
 
 	sig: mount($self, $path, $app)
 
-Adds a new $app to the ecosystem under $path.
+Adds a new $app to the ecosystem under $path. I<$app> can be:
+
+=over
+
+=item
+
+A blessed reference - will try to call run on it
+
+=item
+
+A code reference - will try calling it
+
+=item
+
+A string - will try finding a symbiotic module with that name and mounting it. See L<Kelp::Module::Symbiosis::Base/name>
+
+=back
 
 =head2 run_all
 
 	sig: run_all($self)
 
-DEPRECATED: use L</run> instead.
+I<DEPRECATED in 1.01 use L</run> instead. ETA on removal is no less than three months>
 
 =head2 run
 
 Constructs and returns a new L<Plack::App::URLMap> with all the mounted modules and Kelp itself.
 
+Note: it will not run mounted object twice. This means that it is safe to mount something in two paths at once, and it will just be an alias to the same application.
+
 =head2 mounted
 
 	sig: mounted($self)
 
-Returns a hashref containing a list of mounted modules, keyed with their specified mount paths.
+Returns a hashref containing a list of mounted modules, keyed by their specified mount paths.
+
+=head2 loaded
+
+	sig: loaded($self)
+
+I<new in 1.10>
+
+Returns a hashref containing a list of loaded modules, keyed by their names.
+
+A module is loaded once it is added to Kelp configuration. This can be used to access a module that does not introduce new methods to Kelp.
 
 =head1 METHODS INTRODUCED TO KELP
 
@@ -178,12 +239,36 @@ Shortcut method, same as C<< $kelp->symbiosis->run() >>.
 
 =head1 CONFIGURATION
 
+	# Symbiosis MUST be specified as the first one
+	modules => [qw/Symbiosis Module::Some/],
+	modules_init => {
+		Symbiosis => {
+			mount => '/kelp',
+		},
+		'Module::Some' => {
+			mount => '/some',
+			...
+		},
+	}
+
+Symbiosis should be the first of the symbiotic modules specified in your Kelp configuration. Failure to meet this requirement will cause your application to crash immediately.
+
 =head2 automount
 
+I<DEPRECATED in 1.10: use 'mount' instead. ETA on removal is no less than three months>
+
 Whether to automatically call I<mount> for the Kelp instance, which will be mounted to root path I</>. Defaults to I<1>.
 
 If you set this to I<0> you will have to run something like C<< $kelp->symbiosis->mount($mount_path, $kelp); >> in Kelp's I<build> method. This will allow other paths than root path for the base Kelp application, if needed.
 
+=head2 mount
+
+I<new in 1.10>
+
+A path to mount the Kelp instance, which defaults to I<'/'>. Specify a string if you wish a to use different path. Specify an I<undef> or empty string to avoid mounting at all - you will have to run something like C<< $kelp->symbiosis->mount($mount_path, $kelp); >> in Kelp's I<build> method.
+
+Collides with now deprecated L</automount> - if you specify both, automount will control if the app will be mounted where the I<mount> points to.
+
 =head1 CAVEATS
 
 Routes specified in symbiosis will be matched before routes in Kelp. Once you mount something under I</api> for example, you will no longer be able to specify Kelp route for anything under I</api>.
@@ -194,7 +279,7 @@ Routes specified in symbiosis will be matched before routes in Kelp. Once you mo
 
 =item * L<Kelp::Module::Symbiosis::Base>, a base for symbiotic modules
 
-=item * L<Kelp::Module::Websocket::AnyEvent>, a reference symbiotic module
+=item * L<Kelp::Module::WebSocket::AnyEvent>, a reference symbiotic module
 
 =item * L<Plack::App::URLMap>, Plack URL mapper application
 
