@@ -1,12 +1,16 @@
 package Kelp::Module::Symbiosis::Base;
 
-our $VERSION = '1.01';
+our $VERSION = '1.10';
 
 use Kelp::Base qw(Kelp::Module);
 use Plack::Util;
 
 attr "-middleware" => sub { [] };
 
+# should likely be overriden for a more suitable name
+# won't break backcompat though
+sub name { ref shift }
+
 sub run
 {
 	my ($self) = shift;
@@ -33,11 +37,17 @@ sub build
 {
 	my ($self, %args) = @_;
 
+	die 'Kelp::Module::Symbiosis needs to be loaded before ' . ref $self
+		unless $self->app->can('symbiosis');
+
 	my $middleware = $self->middleware;
 	foreach my $mw (@{$args{middleware}}) {
 		my $config = $args{middleware_init}{$mw};
 		push @$middleware, [$mw, $config];
 	}
+
+	$self->app->symbiosis->_link($self->name, $self, $args{mount});
+	return;
 }
 
 1;
@@ -69,7 +79,7 @@ Kelp::Module::Symbiosis::Base - Base class for symbiotic modules
 
 =head1 DESCRIPTION
 
-This class serves as a base for a Kelp module that is supposed to be ran as a standalone Plack application (mounted separately). It takes care of middleware management, mounting into Symbiosis manager and some basic initialization chores. To write a new module that introduces a standalone Plack application as a Kelp module, simply extend this class and override I<psgi> and I<build> methods.
+This class serves as a base for a Kelp module that is supposed to be ran as a standalone Plack application (mounted separately). It takes care of middleware management, mounting into Symbiosis manager and some basic initialization chores. To write a new module that introduces a standalone Plack application as a Kelp module, simply extend this class and override methods: C<psgi build name> (see below for details).
 
 =head2 Purpose
 
@@ -89,12 +99,26 @@ Calls I<psgi()> and wraps its contents in middlewares. Returns a Plack applicati
 
 By default, this method will throw an exception. It has to be replaced with an actual application producing code in the child class. The resulting application will be wrapped in middlewares from config in I<run()>.
 
+B<Must be reimplemented> in a module.
+
 =head2 build
 
 	sig: build($self, %args)
 
 Standard Kelp module building method. When reimplementing it's best to call parent's implementation, as middleware initialization happens in base implementation.
 
+B<Should be reimplemented> in a module. If it isn't, no extra methods will be added to the Kelp instance, but all the middleware and module registration in Symbiosis will happen anyway.
+
+=head2 name
+
+	sig: name($self)
+
+I<new in 1.10>
+
+Returns a name of a module - a string. This name will be available in L<Kelp::Module::Symbiosis/loaded> hash as a key, containing the module instance as a value.
+
+B<Should be reimplemented> in a module. If it isn't, it will return the name of the package.
+
 =head2 middleware
 
 	sig: middleware($self)
@@ -108,7 +132,7 @@ example configuration could look like this (for L<Kelp::Module::WebSocket::AnyEv
 	modules => [qw/JSON Symbiosis WebSocket::AnyEvent/],
 	modules_init => {
 		Symbiosis => {
-			automount => 0, # kelp will be mounted manually under different path
+			mount => undef, # kelp will be mounted manually under different path
 		},
 		"WebSocket::AnyEvent" => {
 			serializer => "json",
@@ -121,7 +145,20 @@ example configuration could look like this (for L<Kelp::Module::WebSocket::AnyEv
 
 =head2 middleware, middleware_init
 
-Middleware specs for this application. Every module basing on this class can specify its own set of middlewares. They are configured exactly the same as middlewares in Kelp. There's currently no standarized way to retrieve middleware configurations from Kelp into another application, so custom code is needed if such need arise.
+Middleware specs for this application - see above example. Every module basing on this class can specify its own set of middlewares. They are configured exactly the same as middlewares in Kelp. There's currently no standarized way to retrieve middleware configurations from Kelp into another application (to wrap that application in the same middleware as Kelp), so custom code is needed if such need arise.
+
+=head2 mount
+
+	modules_init => {
+		"Symbiotic::Module" => {
+			mount => '/path',
+			...
+		},
+	}
+
+I<new in 1.10>
+
+Should be a string value. If specified, the module will be automatically mounted under that path - there will be no need to call that explicitly, and it will work like: C<< $kelp->symbiosis->mount($path => $module); >>.
 
 =head1 SEE ALSO
 
